# pip install pyqt5-tools
# pip install PyQt5
# pip install PyQt5Designer
# pip install stegano
# pip install plyer
# pip install cryptography
# pip install Pillow
# pyuic5 steg_tool.ui -o steg_tool.py

import os.path
from os import system
from platform import system as psystem
from sys import argv, exit

from cryptography.fernet import Fernet
from PyQt5.QtWidgets import QFileDialog
from plyer import notification

import hex_stg_func
from stegopy import encrypt, decrypt
from steg_tool import *


class MyWin(QtWidgets.QMainWindow):
    def __init__(self, parent=None):
        QtWidgets.QWidget.__init__(self, parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.ui.imgOpenBtn.clicked.connect(self.img_open)
        self.ui.takeBtn.clicked.connect(self.get_job)
        self.ui.savehsText.clicked.connect(self.save_text)
        self.ui.keyBtn.clicked.connect(self.key_path)
        self.ui.hexexCombo.currentTextChanged.connect(self.hexex_change)
        self.ui.wreadCmb.currentTextChanged.connect(self.hexex_change)

    def hexex_change(self):
        if self.ui.hexexCombo.currentText() == "LSB" and self.ui.wreadCmb.currentText() == "Write":
            self.ui.balanceBox.setEnabled(True)
            self.ui.keyBtn.setEnabled(False)
        elif self.ui.hexexCombo.currentText() == "LSB" and self.ui.wreadCmb.currentText() == "Read":
            self.ui.balanceBox.setEnabled(False)
            self.ui.keyBtn.setEnabled(True)
        elif self.ui.hexexCombo.currentText() == "HEX" or self.ui.hexexCombo.currentText() == "ExifHeaders":
            self.ui.keyBtn.setEnabled(False)

    def key_path(self):
        self.ui.pathKey.setText('')
        if psystem() == "Linux":
            file = QtWidgets.QFileDialog.getOpenFileName(self, "Выбор ключа .dat", None,
                                                         "Key file (*.dat)")[0]
        elif psystem() == "Windows":
            file = QtWidgets.QFileDialog.getOpenFileName(self, "Выбор ключа .dat", None,
                                                         "Key file (*.dat)")[0].replace('/', '\\')
        if file == '':
            return

        self.ui.pathKey.setText(file)

    def img_open(self):
        self.ui.savehsText.setEnabled(False)
        self.ui.txtSteg.setText('')
        self.ui.pathImg.setText('')
        if self.ui.hexexCombo.currentText() == "HEX" or self.ui.hexexCombo.currentText() == "ExifHeaders":
            extension = '*.jpg'
        else:
            extension = '*.png *.jpg'

        if psystem() == "Linux":
            file = QtWidgets.QFileDialog.getOpenFileName(self, "Выбор файла", None,
                                                         f"Image file ({extension})")[0]
        elif psystem() == "Windows":
            file = QtWidgets.QFileDialog.getOpenFileName(self, "Выбор файла", None,
                                                         f"Image file ({extension})")[0].replace('/', '\\')
        if file == '':
            return

        if self.ui.wreadCmb.currentText() == "Write":
            self.ui.txtSteg.setEnabled(True)
        self.ui.pathImg.setText(file)
        self.ui.takeBtn.setEnabled(True)

    def get_job(self):
        if self.ui.hexexCombo.currentText() == "HEX":
            self.ui.keyBtn.setEnabled(False)
            if self.ui.wreadCmb.currentText() == "Write":
                if self.ui.txtSteg.toPlainText():
                    hex_stg_func.write_text_ph(self.ui.pathImg.text(), self.ui.txtSteg.toPlainText())
                    self.ui.pathImg.setText('')
                    self.ui.txtSteg.setText('')
                    self.ui.txtSteg.setEnabled(False)
                    self.ui.takeBtn.setEnabled(False)
            elif self.ui.wreadCmb.currentText() == "Read":
                msg = hex_stg_func.read_text_ph(self.ui.pathImg.text())
                if msg == 'В картинке нет текста!':
                    if psystem() == "Windows":
                        notification.notify(message=msg)
                    elif psystem() == "Linux":
                        command = f'''notify-send {msg}"'''
                        system(command)
                    self.ui.txtSteg.setText('')
                    self.ui.takeBtn.setEnabled(False)
                else:
                    self.ui.txtSteg.setText(msg)
                    self.ui.pathImg.setText('')
                    self.ui.takeBtn.setEnabled(False)
                    self.ui.savehsText.setEnabled(True)

        if self.ui.hexexCombo.currentText() == "ExifHeaders":
            self.ui.keyBtn.setEnabled(False)
            if self.ui.wreadCmb.currentText() == "Write":
                if self.ui.txtSteg.toPlainText():
                    path_image_text = os.path.join(os.getcwd(),
                                                   f'{os.path.split(self.ui.pathImg.text())[1].split(".")[0]}_exif.jpg')
                    hex_stg_func.write_text_exif(self.ui.pathImg.text(), path_image_text, self.ui.txtSteg.toPlainText())
                    self.ui.pathImg.setText('')
                    self.ui.txtSteg.setText('')
                    self.ui.txtSteg.setEnabled(False)
                    self.ui.takeBtn.setEnabled(False)
            elif self.ui.wreadCmb.currentText() == "Read":
                msg = hex_stg_func.read_text_exif(self.ui.pathImg.text())
                if msg == 'В картинке нет текста!':
                    if psystem() == "Windows":
                        notification.notify(message=msg)
                    elif psystem() == "Linux":
                        command = f'''notify-send {msg}"'''
                        system(command)
                    self.ui.txtSteg.setText('')
                    self.ui.takeBtn.setEnabled(False)
                else:
                    self.ui.txtSteg.setText(msg)
                    self.ui.pathImg.setText('')
                    self.ui.takeBtn.setEnabled(False)
                    self.ui.savehsText.setEnabled(True)

        if self.ui.hexexCombo.currentText() == "LSB" and self.ui.wreadCmb.currentText() == "Write":
            if self.ui.txtSteg.toPlainText():
                encrypt(self.ui.pathImg.text(), self.ui.txtSteg.toPlainText(), Fernet.generate_key().decode(),
                        int(self.ui.balanceBox.currentText()))
                self.ui.balanceBox.setEnabled(False)
                self.ui.pathImg.setText('')
                self.ui.takeBtn.setEnabled(False)
                self.ui.txtSteg.setText('')
                self.ui.txtSteg.setEnabled(False)
        elif self.ui.hexexCombo.currentText() == "LSB" and self.ui.wreadCmb.currentText() == "Read":
            if self.ui.pathKey.text():
                with open(self.ui.pathKey.text(), 'r') as k:
                    key = k.read()
                dec = decrypt(self.ui.pathImg.text(), key)
                if dec is None:
                    if psystem() == "Windows":
                        notification.notify(message="В файле нет текста или неверный ключ!")
                    elif psystem() == "Linux":
                        command = f'''notify-send "В файле нет текста или неверный ключ!"'''
                        system(command)
                    self.ui.savehsText.setEnabled(False)
                    self.ui.takeBtn.setEnabled(False)
                else:
                    self.ui.txtSteg.setText(dec)
                    self.ui.pathImg.setText('')
                    self.ui.takeBtn.setEnabled(False)
                    self.ui.pathKey.setText('')
                    self.ui.savehsText.setEnabled(True)

    def save_text(self):
        ok = QFileDialog.getSaveFileName(self, "Сохранить файл", ".", "(*.txt)")
        path_s = ok[0].replace("/", "\\") + ok[1]
        if path_s:
            with open(path_s, 'w', encoding='utf-8') as text:
                text.write(self.ui.txtSteg.toPlainText())
            if psystem() == "Windows":
                notification.notify(message='Текст сохранен!')
            elif psystem() == "Linux":
                command = f'''notify-send "Текст сохранен!"'''
                system(command)
            self.ui.txtSteg.setText('')
            self.ui.savehsText.setEnabled(False)


if __name__ == '__main__':
    app = QtWidgets.QApplication(argv)
    myapp = MyWin()
    myapp.show()
    exit(app.exec_())
